@@ -139,6 +139,29 @@ vector<string> DriverIngestor(int nNumberofArgs,char *argv[])
   path_and_file.push_back(path_name);
   path_and_file.push_back(file_name);
 
+  if (path_and_file[0] == "-h" || path_and_file[0] == "--h" || 
+      path_and_file[0] == "--help" || path_and_file[0] == "-help"  ||
+      path_and_file[0] == "help" || path_and_file[0] == "Help"  ||
+      path_and_file[0] == "HELP" || path_and_file[0] == "-help"  ||
+      path_and_file[0] == "-WTF" ||
+      path_and_file[1] == "-h" || path_and_file[1] == "--h" || 
+      path_and_file[1] == "--help" || path_and_file[1] == "-help"  ||
+      path_and_file[1] == "help" || path_and_file[1] == "Help"  ||
+      path_and_file[1] == "HELP" || path_and_file[1] == "-help"  ||
+      path_and_file[1] == "-WTF")
+  {
+    cout << "You have chosen to print the help file. " << endl;
+    cout << "You can find this with README in the filename." << endl;
+    path_and_file[0] = "./";
+    path_and_file[1] = "cry_for_help.txt";
+
+    ofstream ofs;
+    ofs.open("./cry_for_help.txt");
+    ofs << "# The user has cried for help. " << endl;
+    ofs << "# You can find the help file with README in the filename." << endl;
+    ofs.close();
+  }
+
   return path_and_file;
 
 }
@@ -494,6 +517,9 @@ void LSDParameterParser::parse_all_parameters(map<string,float> default_map_f,
 
 }
 
+
+
+
 //-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // This parses all the default parameter maps.
 //-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
@@ -510,6 +536,49 @@ void LSDParameterParser::parse_all_parameters(map<string,float> default_map_f,
 
 }
 
+
+void LSDParameterParser::print_help(map< string, vector<string> > help_map, string help_prefix)
+{
+  
+  // Open the help file
+  ofstream help_ofs;
+  string help_fname = write_path+help_prefix+".csv";
+  cout << "Printing the help file to: " << endl;
+  cout << help_fname << endl;
+  help_ofs.open(help_fname.c_str());
+  vector<string> this_string_vec;
+
+  help_ofs << "name,type,default,description,guidance" << endl;
+
+  // get the parameters
+  vector<string> these_keys = extract_keys(parameters_read_map);
+  //cout << "The number of read keys are: " << these_keys.size() << endl;
+
+  vector<string> more_keys = extract_keys(defaults_used_map);
+  //cout << "The number of defaults are: " << more_keys.size() << endl;
+
+  these_keys.insert( these_keys.end(), more_keys.begin(), more_keys.end() );
+  //cout << "The updated size is: " << these_keys.size() << endl;
+  for(int i = 0; i< int(these_keys.size()); i++)
+  {
+    //cout << "The key is: " <<  these_keys[i] << endl;
+    if ( help_map.find( these_keys[i] ) == help_map.end() ) 
+    {
+      help_ofs << these_keys[i] << ",not available,not available,not available,not available" << endl;
+    } 
+    else 
+    {
+      this_string_vec = help_map[ these_keys[i] ];
+      help_ofs << these_keys[i] << "," << this_string_vec[0] << ","
+               << this_string_vec[1] << "," << this_string_vec[2] << ","
+               << this_string_vec[3] << endl;
+    }
+
+  }
+
+  help_ofs.close();
+}
+
 //-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 // These two functions takes a map of defualt parameters and returns the parameters for the
 // current implementation
@@ -524,17 +593,17 @@ void LSDParameterParser::parse_float_parameters(map<string,float> default_map)
   int n_keys = int(these_keys.size());
   for(int i = 0; i<n_keys; i++)
   {
-    cout << "Key is: " << these_keys[i] << endl;
+    //cout << "Key is: " << these_keys[i] << endl;
 
     // If the key is contained in the parsed parameters, use the parsed parameter
     if(parameter_map.find(these_keys[i]) != parameter_map.end())
     {
-      cout << "Found key " << these_keys[i];
+      //cout << "Found key " << these_keys[i];
 
       // convert the value to float
       float_parameters[these_keys[i]] = atof(parameter_map[these_keys[i]].c_str());
       parameters_read_map[these_keys[i]] = parameter_map[these_keys[i]];
-      cout << " it is: " << parameter_map[these_keys[i]] << " check: " << float_parameters[these_keys[i]] << endl;
+      //cout << " it is: " << parameter_map[these_keys[i]] << " check: " << float_parameters[these_keys[i]] << endl;
 
     }
     else  // the key is not in the parsed parameters. Use the default.
@@ -560,17 +629,17 @@ void LSDParameterParser::parse_double_parameters(map<string,double> default_map)
   int n_keys = int(these_keys.size());
   for(int i = 0; i<n_keys; i++)
   {
-    cout << "Key is: " << these_keys[i] << endl;
+    //cout << "Key is: " << these_keys[i] << endl;
 
     // If the key is contained in the parsed parameters, use the parsed parameter
     if(parameter_map.find(these_keys[i]) != parameter_map.end())
     {
-      cout << "Found key " << these_keys[i];
+      //cout << "Found key " << these_keys[i];
 
       // convert the value to float
       double_parameters[these_keys[i]] = atof(parameter_map[these_keys[i]].c_str());
       parameters_read_map[these_keys[i]] = parameter_map[these_keys[i]];
-      cout << " it is: " << parameter_map[these_keys[i]] << " check: " << double_parameters[these_keys[i]] << endl;
+      //cout << " it is: " << parameter_map[these_keys[i]] << " check: " << double_parameters[these_keys[i]] << endl;
 
     }
     else  // the key is not in the parsed parameters. Use the default.
@@ -593,7 +662,7 @@ void LSDParameterParser::parse_int_parameters(map<string,int> default_map)
   int n_keys = int(these_keys.size());
   for(int i = 0; i<n_keys; i++)
   {
-    cout << "Key is: " << these_keys[i] << endl;
+    //cout << "Key is: " << these_keys[i] << endl;
 
     // If the key is contained in the parsed parameters, use the parsed parameter
     if(parameter_map.find(these_keys[i]) != parameter_map.end())
@@ -623,7 +692,7 @@ void LSDParameterParser::parse_bool_parameters(map<string,bool> default_map)
   int n_keys = int(these_keys.size());
   for(int i = 0; i<n_keys; i++)
   {
-    cout << "Key is: " << these_keys[i] << endl;
+    //cout << "Key is: " << these_keys[i] << endl;
 
     // If the key is contained in the parsed parameters, use the parsed parameter
     if(parameter_map.find(these_keys[i]) != parameter_map.end())
@@ -660,7 +729,7 @@ void LSDParameterParser::parse_string_parameters(map<string,string> default_map)
   int n_keys = int(these_keys.size());
   for(int i = 0; i<n_keys; i++)
   {
-    cout << "Key is: " << these_keys[i] << endl;
+    //cout << "Key is: " << these_keys[i] << endl;
 
     // If the key is contained in the parsed parameters, use the parsed parameter
     if(parameter_map.find(these_keys[i]) != parameter_map.end())
