@@ -1827,14 +1827,14 @@ void LSDSpatialCSVReader::interpolate_across_gap(LSDRaster& ThisRaster, string X
       ThisRaster.get_x_and_y_locations(row-1, col, s_x, s_y);
       ThisRaster.get_x_and_y_locations(row-1, col+1, se_x, se_y);
 
-      if((next_x == nw_x and next_y == nw_y)
-      || (next_x == n_x and next_y == n_y)
-      || (next_x == ne_x and next_y == ne_y)
-      || (next_x == e_x and next_y == e_y)
-      || (next_x == w_x and next_y == w_y)
-      || (next_x == sw_x and next_y == sw_y)
-      || (next_x == s_x and next_y == s_y)
-      || (next_x == se_x and next_y == se_y)
+      if((next_x == nw_x && next_y == nw_y)
+      || (next_x == n_x && next_y == n_y)
+      || (next_x == ne_x && next_y == ne_y)
+      || (next_x == e_x && next_y == e_y)
+      || (next_x == w_x && next_y == w_y)
+      || (next_x == sw_x && next_y == sw_y)
+      || (next_x == s_x && next_y == s_y)
+      || (next_x == se_x && next_y == se_y)
       )
       {
         // cout << "We've got the next node, all good" << endl;
